#include "Challenges.h"

#include "ChallengeHandlers.h"

#include <QSettings>

IOHandler* Challenge::createHandler(BrainFunk* bf) const
{
  if (testMode == "normal")
  {
    NormalTestHandler* normal = new NormalTestHandler(bf, data);
    return normal;
  }
  else if (testMode == "alpha")
  {
    AlphaTestHandler* alpha = new AlphaTestHandler(bf, data);
    return alpha;
  }
  else if (testMode == "nim")
  {
    NimTestHandler* nim = new NimTestHandler(bf);
    return nim;
  }

  Q_ASSERT(false);
  return NULL;
}

void ChallengeLoader::load()
{
  global::challenges.clear();

  if (!readNextStartElement())
    return;

  if (name() != "xml")
    return;

  while (readNextStartElement())
  {
    if (name() == "challenge")
      global::challenges += loadChallenge();
    else
      skipCurrentElement();
  }
}

Challenge ChallengeLoader::loadChallenge()
{
  Challenge result;

  while (readNextStartElement())
  {
    if (name() == "name")
      result.name = readElementText();
    else if (name() == "text")
      result.description = readElementText();
    else if (name() == "difficulty")
      result.difficulty = readElementText().toInt();
    else if (name() == "tape_length")
      result.tapeLength = readElementText().toInt();
    else if (name() == "program_length")
      result.programLength = readElementText().toInt();
    else if (name() == "test")
    {
      result.testMode = attributes().value("type").toString();

      while (readNextStartElement())
      {
        if (name() == "data")
          result.data += readElementText();
        else
          skipCurrentElement();
      }
    }
    else
      skipCurrentElement();
  }

  return result;
}

namespace global
{

QList<Challenge> challenges;

bool isChallengeComplete(int number)
{
  QSettings settings;
  settings.beginReadArray("challenge");
  settings.setArrayIndex(number);
  bool result = settings.value("complete", false).toBool();
  settings.endArray();

  return result;
}

void setChallengeComplete(int number)
{
  QSettings settings;
  settings.beginWriteArray("challenge");
  settings.setArrayIndex(number);
  settings.setValue("complete", true);
  settings.endArray();
}

}
