#ifndef Machine_h
#define Machine_h

#include <QString>

class IOHandler;

class Machine
{
public:
  enum HaltCode
  {
    HaltNone,

    HaltNormal,
        
    HaltOverLeft,
    HaltOverRight,
    HaltBadInput,
    HaltBadOutput,
    HaltUnmatchedLoop,
    HaltUnfinishedResult,

    HaltUnknown,
  };

  enum
  {
    DefaultMaximum = 33713, // Why this number? Why not?
  };
  
  Machine();
  Machine(int tapeSize);
  
  ~Machine();

  void setIOHandler(IOHandler* handler);
  
  int tapeSize() const;
  void setTapeSize(int size);
  
  int maximumValue() const;
  void setMaximumValue(int max);
  
  void setProgram(const QString& program);
  void reset();
  void clear();
  bool step();
  
  HaltCode haltCode() const;

  int programPosition() const;
  QChar programValue() const;
  int tapePosition() const;
  int tapeValue(int position) const;
  int tapeValue() const
  {
    return tapeValue(tapePosition());
  }

  int findLoopExit(int position) const;

private:
  QChar readCode();
  void halt(HaltCode code);

  struct Private;
  Private* d;
};

#endif
