#include "TapeModel.h"
#include  "Machine.h"

#include <QIcon>
#include <QFont>
#include <QPalette>

TapeModel::TapeModel(QObject* parent) :
  QAbstractItemModel(parent),
  machine(NULL)
{
}

TapeModel::TapeModel(Machine* machine_, QObject* parent) :
  QAbstractItemModel(parent)
{
  setMachine(machine_);
}

TapeModel::~TapeModel()
{
}

void TapeModel::setMachine(Machine* machine_)
{
  machine = machine_;
  reset();
}

int TapeModel::columnCount(const QModelIndex&) const
{
  return machine->tapeSize();
}

QModelIndex TapeModel::positionIndex() const
{
  int target = qBound(0, machine->tapePosition(), machine->tapeSize() - 1);
  return createIndex(0, target);
}

QVariant TapeModel::data(const QModelIndex& index, int role) const
{
  int row = index.row();
  if (row != 0)
    return QVariant();

  int cell = index.column();
  if (cell < 0 || cell >= machine->tapeSize())
    return QVariant();

  if (role == Qt::BackgroundRole && (cell & 1))
  {
    QColor base = QPalette().base().color();
    if (cell & 2)
      return base.darker(118);
    return base.darker(112);
  }

  if (role == Qt::DisplayRole)
    return QString::number(machine->tapeValue(cell));

  // After here just tidy up the tape head cell
  if (cell != machine->tapePosition())
      return QVariant();

  if (role == Qt::DecorationRole)
    return QIcon(":/check.png");

  if (role == Qt::FontRole)
  {
    QFont font;
    font.setBold(true);
    return font;
  }

  return QVariant();
}
